/*
 * Alert handling system.
 *
 * Usage:
 * Alert.ERROR(String message) for errors
 * Alert.INFO(String message) for information
 * Alert.WARNING(String message) for warnings
 * Alert.QUESTION(String message) for problems/questions
 *
 * No need for initialization.
 *
 * Everything is done within NOTIFY subroutine, although others are created for
 * easier implementation in code.
 *
 */
import javax.swing.*;

class Alert {
    
    private static void NOTIFY (String message, int type) {
        
        String typeName;
        
        switch (type) {
            case JOptionPane.ERROR_MESSAGE:
                typeName = "Error";
                break;
            case JOptionPane.INFORMATION_MESSAGE:
                typeName = "Information";
                break;
            case JOptionPane.WARNING_MESSAGE:
                typeName = "Warning";
                break;
            case JOptionPane.QUESTION_MESSAGE:
                typeName = "Question";
                break;
            default:
                typeName = "Unidentified";
                type = JOptionPane.INFORMATION_MESSAGE;
        }
        
        JOptionPane.showMessageDialog(null, message, typeName, type);
    }
    
    // Shortcut subroutines:
    
    public static void ERROR (String message)
        { NOTIFY(message, JOptionPane.ERROR_MESSAGE); }
    
    public static void ERROR (String message, Exception exception)
        { NOTIFY(message+"\n ERROR MESSAGE:\n"+exception, JOptionPane.ERROR_MESSAGE); }
    
    public static void INFO (String message)
        { NOTIFY(message, JOptionPane.INFORMATION_MESSAGE); }
    
    public static void WARNING (String message)
        { NOTIFY(message, JOptionPane.WARNING_MESSAGE); }
    
    public static void QUESTION (String message)
        { NOTIFY(message, JOptionPane.QUESTION_MESSAGE); }
   
}