/*
 * Class: ChordBookMenu()
 *
 * Creates common application menu. 
 * Invoked using sinleton pattern ChordBookMenu.getInstance();
 * No need to initialize.
 *
 * Dependant directly to ChordBook application
 *
 */

import java.awt.Toolkit;
import javax.swing.*;
import java.awt.event.*;

class ChordBookMenu extends JMenuBar implements ActionListener {
    
    private JSeparator Separator1 = new JSeparator();
    private JSeparator Separator2 = new JSeparator();
    
    private JMenu FileMenu = new JMenu("File");
    private JMenuItem NewItem = new JMenuItem("New", new ImageIcon("./icons/document-new.png"));
    private JMenuItem OpenItem = new JMenuItem("Open", new ImageIcon("./icons/document-open.png"));
    private JMenuItem DuplicateItem = new JMenuItem("Duplicate", new ImageIcon("./icons/document-new.png"));
    private JMenuItem SaveItem = new JMenuItem("Save", new ImageIcon("./icons/document-save.png"));
    private JMenuItem DeleteItem = new JMenuItem("Delete", new ImageIcon("./icons/edit-delete.png"));
    private JMenuItem PrintItem = new JMenuItem("Print", new ImageIcon("./icons/document-print.png"));
    
    private JMenu EditMenu = new JMenu("Edit");
    private JMenuItem TagsItem = new JMenuItem("Edit tags", new ImageIcon("./icons/accessories-text-editor.png"));
    
    private JMenu ViewMenu = new JMenu("View");
    private JCheckBoxMenuItem WrapText = new JCheckBoxMenuItem("Wrap text");
    
    private JMenu ToolsMenu = new JMenu("Tools");
    private JMenuItem CDBMenuItem = new JMenuItem("Chords Database", new ImageIcon("./icons/applications-other.png"));
    private ChordDB ChordDB;

    
    private ChordBookMenu() {
        super();
        
        this.add(FileMenu);
        FileMenu.add(NewItem);
        FileMenu.add(OpenItem);
        FileMenu.add(Separator1);
        FileMenu.add(DuplicateItem);
        FileMenu.add(SaveItem);
        SaveItem.setActionCommand("Save");
        SaveItem.addActionListener(this);
        SaveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        FileMenu.add(DeleteItem);
        FileMenu.add(Separator2);
        FileMenu.add(PrintItem);
        
        SaveItem.setEnabled(false);
        
        this.add(EditMenu);
        EditMenu.add(TagsItem);
        TagsItem.setActionCommand("EditTags");
        TagsItem.addActionListener(this);
        TagsItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        
/*        this.add(ViewMenu);
        ViewMenu.add(WrapText);
        WrapText.setActionCommand("WrapText");
        WrapText.addActionListener(this);
  */      
        this.add(ToolsMenu);
        ToolsMenu.add(CDBMenuItem);
        CDBMenuItem.setActionCommand("ChordDB");
        CDBMenuItem.addActionListener(this);

    }
    
    // Singleton pattern
    // to get only ONE instance of menu always
    public static ChordBookMenu getInstance() {
        return ChordBookMenuHolder.instance;
    }
    
    private static class ChordBookMenuHolder {
           private static ChordBookMenu instance = new ChordBookMenu();
    }
    
    public void SaveItemState(boolean s) {
        SaveItem.setEnabled(s);
    }
    
    public void actionPerformed(ActionEvent e) {
        
        // menu actions
        if( e.getActionCommand() == "ChordDB") {
            //ChordDB = ChordDB.getInstance();
            //ChordDB.show();
            CDBInterface i = new CDBInterface();
            i.show();
        }
        
        else if ( e.getActionCommand() == "EditTags" ) {
            // Using Singleton pattern to call to main window
            
            ChordBookInterface.getInstance().EditTags();
        }
        
        else if ( e.getActionCommand() == "WrapText" ) {
            ChordBookInterface.getInstance().WrapText(WrapText.getState());
        }
        
        else if ( e.getActionCommand() == "Save" )
            ChordBookInterface.getInstance().SaveOpenedSong();
        
        else {
            Alert.ERROR("No actionPerformed() defined");
        }
        
    }
    
}
