/*
 * Class: Config() Config(String filename)
 * Used to parse 'attribute=value' style configuration files
 *
 * Initializing without attributes calls default configuration file
 * Static methods, no need for initialization
 *
 * Methods: String getValue(String attribute)  returns value or NULL
 *          setValue(String attribute, String value)   writes settings to file
 */

import java.io.*;
import java.util.Vector;

class Config {
    
    private static String CONFIG = new String("./chordbook.conf");
    
    Config() { }
    Config(String filename) {
        CONFIG = filename;
    }
    
    public static String getValue(String attribute) {
        String value = new String();
        try {
            BufferedReader file = new BufferedReader(new FileReader(CONFIG));
            String line;

            while ((line = file.readLine())!= null) {
                if (!line.startsWith(";") && line.startsWith(attribute))
                    value = line.substring(line.indexOf("=")+1,line.length());
            }
        }
        catch (IOException e) {
            Alert.ERROR("Cannot read configuration file",e);
        }
        
        return value;
    }
    
    public static void setValue (String attribute, int value) {
        set(attribute, Integer.toString(value));
    }
    public static void setValue (String attribute, String value) {
        set(attribute, value);
    }
    private static void set (String attribute, String value) {
        try {
            BufferedReader fileI = new BufferedReader(new FileReader(CONFIG));
            String line;
            Vector conf = new Vector();
            
            while ((line = fileI.readLine())!= null) {
                String attrib;
                if(line.indexOf("=") > -1)
                    attrib = line.substring(0,line.indexOf("="));
                else
                    attrib = null;
                
                if(!attribute.equals(attrib) || line.startsWith(";"))
                    conf.add(line);
            }
            conf.add(attribute+"="+value);
            fileI.close();
            
            Writer fileO = new BufferedWriter(new FileWriter(CONFIG));

            for(int i=0;i<conf.size();i++)
                fileO.write(conf.get(i).toString()+"\n"); //System.getProperty("line.separator")
            
            fileO.close();

        }
        catch (IOException e) {
            Alert.ERROR("Cannot write configuration file",e);
        }
    }
}
