/*
 *  class: Loader
 *
 *  Used once before loading main interface.
 *  Reads library folder, finds all song files, parses tags and returns song vector
 *  
 *  Method: Vector getSongList()   returns Vector with Song objects
 *
 */

import java.util.Arrays; //for sorting
import java.util.Vector;
import java.io.*;

class Loader {

    private int songcount = 0;
    private Vector song = new Vector();
    public boolean finished = false;
    
    Loader() {
        
        LoaderInterface LoaderInterface = new LoaderInterface();
        
        String[] data = new File(Config.getValue("library")).list();
        Arrays.sort(data);
        
        int step = (100/data.length); // progress in % after parsing 1 song
        
        for (int i=0; i<data.length; i++)
        {
            if (data[i].endsWith(".txt")) {
                song.add(CreateSong(data[i]));
                songcount++;
                LoaderInterface.setProgress(step*songcount,"Parsing: "+data[i]);
            }
        }
        
        LoaderInterface.setProgress(100,"Finished");
        finished = true;
        LoaderInterface.hide();
        
    }
    
    
    
    public Vector getSongList() {
        return song;
    }
    
    private Song CreateSong(String filename) {

        Song sng = new Song(); // creates new song for list
        
        try {
            BufferedReader input = new BufferedReader(new FileReader(Config.getValue("library")+"/"+filename));
            String line;
            boolean stop = false;

            int tline, lineN;
            lineN = 0;
            
            sng.FILENAME = filename;

            while ((line = input.readLine())!= null) {
                
                if (line.length() > 0 && line.startsWith("[")) {
     
                            int side = (line.substring(1,(line.length()-1))).indexOf("=");
                            String tag = line.substring(1,side+1);
                            String att = line.substring(side+2,line.length()-1);
                            tag.toUpperCase(); // converting tags to uppercase to avoid parsing errors
                            if (tag.compareTo("ARTIST") == 0) sng.ARTIST = att;
                            else if (tag.compareTo("TITLE") == 0) sng.TITLE = att;
                            else if (tag.compareTo("ALBUM") == 0) sng.ALBUM = att;
                            else if (tag.compareTo("COMMENTS") == 0) sng.COMMENTS = att;
                            else if (tag.compareTo("TEMPO") == 0) sng.TEMPO = Integer.parseInt(att);
                            else if (tag.compareTo("TRANSPOSE") == 0) sng.TRANSPOSE = Integer.parseInt(att);
                        }
                }

        } catch (IOException E) { System.out.println("Cannot read song: "+E); }
        
        if (sng.TITLE == null || sng.ARTIST == null) {
            TagEditor tags = new TagEditor(sng);
            
            while (!tags.state); //wait until user completes the new dialog
            
            sng = tags.song;
        }
        
        return sng; // returns SONG object
        
    }
}
