/*
 * Class: TagEditor()
 *
 * Interface used to edit tags. Writing to file is done in Song() class.
 * Constructor:  TagEditor(Song s)
 * Given song must be either for entering or updating tags. 
 * Information text is set up accordingly.
 *
 */

import java.awt.Dimension;
import java.awt.Rectangle;

public class TagEditor extends javax.swing.JFrame {
    
    /** Creates new form TagEditor */
    public TagEditor(Song s) {
        song = s;
        initComponents();
        Filename.setText(song.FILENAME);
        
        if(s.ARTIST.compareTo("")!=0 && s.TITLE.compareTo("")!=0) {
            // if EDITING tags
            Information.setText("Update tags for the selected song.");
            Album.setText(s.ALBUM);
            Artist.setText(s.ARTIST);
            Comments.setText(s.COMMENTS);
            TempoSlider.setValue(s.TEMPO);
            Title.setText(s.TITLE);
            TransposeSlider.setValue(s.TRANSPOSE);
        }
        else
            Information.setText("The song file has no tags. Please set the tags.");
        
        Dimension dim = getToolkit().getScreenSize();
        Rectangle abounds = getBounds();
        this.setLocation((dim.width - abounds.width) / 2, (dim.height - abounds.height) / 2);
        
        this.show();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        Information = new javax.swing.JLabel();
        Information2 = new javax.swing.JLabel();
        ControlsPanel = new javax.swing.JPanel();
        FilenameLabel = new javax.swing.JLabel();
        Filename = new javax.swing.JLabel();
        ArtistLabel = new javax.swing.JLabel();
        TitleLabel = new javax.swing.JLabel();
        AlbumLabel = new javax.swing.JLabel();
        TempoLabel = new javax.swing.JLabel();
        TransposeLabel = new javax.swing.JLabel();
        CommentsLabel = new javax.swing.JLabel();
        Artist = new javax.swing.JTextField();
        Title = new javax.swing.JTextField();
        Album = new javax.swing.JTextField();
        TempoPanel = new javax.swing.JPanel();
        TempoSlider = new javax.swing.JSlider();
        TempoBPM = new javax.swing.JLabel();
        TransposePanel = new javax.swing.JPanel();
        TransposeSlider = new javax.swing.JSlider();
        TransposeValue = new javax.swing.JLabel();
        CommentsScroll = new javax.swing.JScrollPane();
        Comments = new javax.swing.JTextArea();
        Save = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        Information.setText("%state information%");

        Information2.setFont(new java.awt.Font("Lucida Grande", 0, 10));
        Information2.setText("* required fields");

        FilenameLabel.setText("Filename:");

        Filename.setText("%filename%");

        ArtistLabel.setText("Artist:");

        TitleLabel.setText("Title:");

        AlbumLabel.setText("Album:");

        TempoLabel.setText("Tempo:");

        TransposeLabel.setText("Transpose:");

        CommentsLabel.setText("Comments:");

        TempoSlider.setMaximum(280);
        TempoSlider.setMinimum(32);
        TempoSlider.setMinorTickSpacing(10);
        TempoSlider.setPaintTicks(true);
        TempoSlider.setValue(120);
        TempoSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                TempoSliderStateChanged(evt);
            }
        });

        TempoBPM.setText("120 bpm");
        TempoBPM.setEnabled(false);

        org.jdesktop.layout.GroupLayout TempoPanelLayout = new org.jdesktop.layout.GroupLayout(TempoPanel);
        TempoPanel.setLayout(TempoPanelLayout);
        TempoPanelLayout.setHorizontalGroup(
            TempoPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, TempoPanelLayout.createSequentialGroup()
                .add(TempoSlider, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 238, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(TempoBPM))
        );
        TempoPanelLayout.setVerticalGroup(
            TempoPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(TempoBPM, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 29, Short.MAX_VALUE)
            .add(TempoSlider, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 29, Short.MAX_VALUE)
        );

        TransposeSlider.setMajorTickSpacing(2);
        TransposeSlider.setMaximum(12);
        TransposeSlider.setMinimum(-12);
        TransposeSlider.setMinorTickSpacing(1);
        TransposeSlider.setPaintTicks(true);
        TransposeSlider.setSnapToTicks(true);
        TransposeSlider.setValue(0);
        TransposeSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                TransposeSliderStateChanged(evt);
            }
        });

        TransposeValue.setText("(0)");
        TransposeValue.setEnabled(false);

        org.jdesktop.layout.GroupLayout TransposePanelLayout = new org.jdesktop.layout.GroupLayout(TransposePanel);
        TransposePanel.setLayout(TransposePanelLayout);
        TransposePanelLayout.setHorizontalGroup(
            TransposePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, TransposePanelLayout.createSequentialGroup()
                .add(TransposeSlider, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 278, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(TransposeValue))
        );
        TransposePanelLayout.setVerticalGroup(
            TransposePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(TransposeValue, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 38, Short.MAX_VALUE)
            .add(TransposeSlider, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        Comments.setColumns(20);
        Comments.setRows(5);
        CommentsScroll.setViewportView(Comments);

        org.jdesktop.layout.GroupLayout ControlsPanelLayout = new org.jdesktop.layout.GroupLayout(ControlsPanel);
        ControlsPanel.setLayout(ControlsPanelLayout);
        ControlsPanelLayout.setHorizontalGroup(
            ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(ControlsPanelLayout.createSequentialGroup()
                .add(23, 23, 23)
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(TempoLabel)
                    .add(AlbumLabel)
                    .add(TitleLabel)
                    .add(ArtistLabel)
                    .add(FilenameLabel)
                    .add(TransposeLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(TempoPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(TransposePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(Album, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE)
                    .add(Title, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE)
                    .add(Filename, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE)
                    .add(Artist, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE)))
            .add(ControlsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(CommentsLabel)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(CommentsScroll, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE))
        );
        ControlsPanelLayout.setVerticalGroup(
            ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(ControlsPanelLayout.createSequentialGroup()
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(FilenameLabel)
                    .add(Filename))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(ArtistLabel)
                    .add(Artist, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(TitleLabel)
                    .add(Title, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(AlbumLabel)
                    .add(Album, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(TempoPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(TempoLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, TransposePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, TransposeLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ControlsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(ControlsPanelLayout.createSequentialGroup()
                        .add(CommentsLabel)
                        .addContainerGap(69, Short.MAX_VALUE))
                    .add(CommentsScroll, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 85, Short.MAX_VALUE)))
        );

        Save.setText("Save");
        Save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SaveActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(ControlsPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(Information2)
                    .add(Information)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, Save))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(Information)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(Information2)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(ControlsPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(Save)
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void SaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SaveActionPerformed
        if (!TempoBPM.isEnabled())    // if tempo isn't given
            TempoSlider.setValue(-1);
        
        song.updateTags(Artist.getText(), Title.getText(), Album.getText(),
                Comments.getText(), TempoSlider.getValue(), TransposeSlider.getValue());
        state = true;
        this.hide();
    }//GEN-LAST:event_SaveActionPerformed

    private void TransposeSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_TransposeSliderStateChanged
        TransposeValue.setEnabled(true);
        TransposeValue.setText(Integer.toString(TransposeSlider.getValue()));
    }//GEN-LAST:event_TransposeSliderStateChanged

    private void TempoSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_TempoSliderStateChanged
        TempoBPM.setEnabled(true);
        TempoBPM.setText(TempoSlider.getValue()+" bpm");
    }//GEN-LAST:event_TempoSliderStateChanged
    
    public Song song = new Song();
    public boolean state = false;
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField Album;
    private javax.swing.JLabel AlbumLabel;
    private javax.swing.JTextField Artist;
    private javax.swing.JLabel ArtistLabel;
    private javax.swing.JTextArea Comments;
    private javax.swing.JLabel CommentsLabel;
    private javax.swing.JScrollPane CommentsScroll;
    private javax.swing.JPanel ControlsPanel;
    private javax.swing.JLabel Filename;
    private javax.swing.JLabel FilenameLabel;
    private javax.swing.JLabel Information;
    private javax.swing.JLabel Information2;
    private javax.swing.JButton Save;
    private javax.swing.JLabel TempoBPM;
    private javax.swing.JLabel TempoLabel;
    private javax.swing.JPanel TempoPanel;
    private javax.swing.JSlider TempoSlider;
    private javax.swing.JTextField Title;
    private javax.swing.JLabel TitleLabel;
    private javax.swing.JLabel TransposeLabel;
    private javax.swing.JPanel TransposePanel;
    private javax.swing.JSlider TransposeSlider;
    private javax.swing.JLabel TransposeValue;
    // End of variables declaration//GEN-END:variables
    
}
