/* 
 * class Chord
 *
 * definition of Chord - Base: A-G[b#] 
 *
 * method isEqual: returns true if objects are equal
 * method compareStrings: error-free comparison of strings (no null pointer exception)
 *
 *
 * class CDBChord (ChordDatabase Chord -> used in CDB for storing Chord with fingering)
 */

import org.jfugue.*;

class CDBChord extends Chord {
    // 30 bites for each fingering = 24(6 digits)+6(6places for X)
    char[] fingering1 = new char[6];
    char[] fingering2 = new char[6];

    public void playChord(char[] fi) {
        
        Player player = new Player();
        player.play(FingeringToPattern(fi));
        
    }
    
    public static Pattern FingeringToPattern(String notes) {
        char[] n = new char[6];
        for(int i=0;i<6;i++)
            n[i] = notes.charAt(i);
        
        return FingeringToPattern(n);
    }
    
    public static Pattern FingeringToPattern(char[] notes) {
        String pat = "I["+Config.getValue("instrument")+"]";
        int[] guitar = {4, 9, 14, 19, 23, 28}; // (guitar[int]+3*12) % 12 = octave
        String[] octave = {"C", "C#", "D", "Eb", "E", "F", "F#", "G", "Ab", "A", "Bb", "B"};

        for(int i=0;i<6;i++) {
            if(notes[i] != 'X' && notes[i] != 'x') {
                int note = (Integer.parseInt(""+notes[i])+guitar[i]);
                pat = pat+" "+octave[(note+12)%12]+(2+note/12);
            }
        }
            
        return new Pattern(pat);
    }
}

class Chord {
    String base, extra;
    
    Chord() { }
    
/*    Chord Chord(Chord c) {
        Chord temp = new Chord();
        temp.base = new String(c.base);
        temp.extra = new String(c.extra);
        return temp;
    }
  */  
    public Chord transpose(int t) {
        char[] bases = {'C',' ','D',' ','E','F',' ','G',' ','A',' ','B'};
        String[] octave = {"C", "C#", "D", "Eb", "E", "F", "F#", "G", "Ab", "A", "Bb", "B"};
        int finalNote = '0';
        if(this.base.length()>1) {
            switch (this.base.charAt(1)) {
                case '#': t++; break;
                case 'b': t--; break;
                default: break;
            }
        }
        for(int i=0;i<12;i++) {
            if(bases[i] == this.base.charAt(0))
            {
                finalNote = (i+t+12)%12;
                break;
            }
        }
        this.base = new String(octave[finalNote]);

        return this;
    }
    
    public String toString() {
        if(extra==null)
            return base;
        else
            return base+extra;
    }
    
    /* Method for comparing chords. Returns boolean.
     */
    public boolean isEqual (Chord C) /* OK */ {
        // print comparison results [t/f]:
        //System.out.print(base+extra+"-"+C.base+C.extra+":");
        //System.out.println(compareStrings(base, C.base)&&compareStrings(extra, C.extra));
        
        return (compareStrings(base, C.base)&&compareStrings(extra, C.extra));
    }

    private boolean compareStrings(String A, String B) /* OK */ {
        /* Exended string compare method. Required as chord optional part (extra) may be NULL.
         * Method executed two times for every chord: for base & extra part separately.
         */
        if (A==null && B==null)
            return true;
        else if ((A==null && B!=null) || (A!=null && B==null))
            return false;
        else if ((A!=null && B!=null) && A.compareTo(B) == 0)
            return true;
        else
            return false;
    }
 }

/*
 * class LChord
 * Object for LinkedChordList containing Chord, link to previous/next LChord
 */

class LChord {
    LChord previous, next;
    Chord chord;
    public LChord () {}
    public LChord (Chord ch) {
        previous = null;
        next = null;
        chord = ch;
    }
    
    public LChord (Chord ch, LChord pr) {
        previous = pr;
        next = null;
        chord = ch;
    }
    
    public LChord (Chord ch, LChord pr, LChord ne) {
        previous = pr;
        next = ne;
        chord = ch;
    }
}

class LinkedChordList {
    
    LChord first, last;
    
    LinkedChordList () {
        first = null;
        last = null;
    }
    
    void print () {
        LChord current = first;
        System.out.print("\nFinal list: ");
        while (current != null) {
            System.out.print(current.chord.base);
            if (current.chord.extra != null)
                System.out.print(current.chord.extra);
            System.out.print(" ");
            current = current.next;
            
        }
        System.out.println("<<");
    }
    
    // Chord Search. Returns list item.
    LChord search (Chord s) /* OK */ {
        return searchNext(s, first);
    }
    
    LChord searchNext (Chord what, LChord where) /* OK */ {
        if (where == null)             // empty list or finished searching
            return null;
        
        if (what.isEqual(where.chord)) // equal element found
            return where;
        else
            return searchNext (what, where.next); // proceed to next element in list
    }
    
    void add (Chord ch) /* OK */{
        // Add if chord is unique in the list
        LChord chord;
        if (first == null) {
            chord = new LChord(ch);
            first = chord;
            last = chord;
        }
        else {
            if (search(ch)==null) {
                chord = new LChord(ch, last); // creating new node with pointer THIS.PREVIOUS -> LAST
                last.next = chord;            // assigning pointer for LAST.next -> THIS
                last = chord;                 // assigning THIS as LAST node in list
            }
        }
        //print(); // print out list after addition. should be no duplicates // done in parseSong
    }
}

/*
 * linked list methods:
 * size isEmpty 
first last 
before after 
insertHead insertTail 
insertAfter insertBefore
 **/