import java.io.*;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.text.*;

/*
 *  SONG object. Contains TITLE, ARTIST, ALBUM, COMMENTS, FILENAME and TEMPO
 *  that are assigned during getDirContents() instance once the application is
 *  launched.
 *
 */

class Song {
    
    String TITLE;
    String ARTIST;
    String ALBUM;
    String COMMENTS;
    String FILENAME;
    int TRANSPOSE;
    int TEMPO;
    LinkedChordList ChordList;
    private Document doc;

    public Document getDocument() {
        return getDoc(0);
    }
    
    public Document getDocument(int t) {
        return getDoc(t);
    }
    
    private Document getDoc(int transp) {

        TRANSPOSE = transp;
        try {
            return parseSong();
        }
        catch (IOException e) {
            Alert.ERROR("Error in getDocument()",e);
            return null;
        }
    }
    
    public void saveDocument(Document d) {
        try {
            BufferedReader fileI = new BufferedReader(new FileReader(Config.getValue("library")+"/"+this.FILENAME));
            String line;
            Vector songFile = new Vector();
            
            while ((line = fileI.readLine())!= null)
                if(line.startsWith("[") && line.endsWith("]")) // leave tags
                    songFile.add(line);
            
            try {
                songFile.add(d.getText(0, d.getLength()));
            } catch (BadLocationException e) {
                Alert.ERROR("Bad location", e); 
            }
                    
            fileI.close();
            
            // ^ Songfile content ready
            
            Writer fileO = new BufferedWriter(new FileWriter(Config.getValue("library")+"/"+this.FILENAME));

            for(int i=0;i<songFile.size();i++)
                fileO.write(songFile.get(i).toString()+"\n"); //System.getProperty("line.separator")
            
            fileO.close();
        }
        catch (IOException e) {
            Alert.ERROR("Cannot save document!",e);
        }
    }
    
    public void updateTags(String artist, String title, String album, String comments, int tempo, int transpose) {
        try {
            BufferedReader fileI = new BufferedReader(new FileReader(Config.getValue("library")+"/"+this.FILENAME));
            String line;
            Vector songFile = new Vector();
            
            songFile.add("["+"ARTIST="+artist+"]");
            songFile.add("["+"TITLE="+title+"]");
            songFile.add("["+"ALBUM="+album+"]");
            songFile.add("["+"COMMENTS="+comments+"]");
            
            if(tempo > -1)
                songFile.add("["+"TEMPO="+tempo+"]");
            
            songFile.add("["+"TRANSPOSE="+transpose+"]");
            
            while ((line = fileI.readLine())!= null)
                if(!line.startsWith("[") && !line.endsWith("]")) // remove old tags
                    songFile.add(line);
                
            fileI.close();
            
            // ^ Songfile content ready
            
            Writer fileO = new BufferedWriter(new FileWriter(Config.getValue("library")+"/"+this.FILENAME));

            for(int i=0;i<songFile.size();i++)
                fileO.write(songFile.get(i).toString()+"\n"); //System.getProperty("line.separator")
            
            fileO.close();

        }
        catch (IOException e) {
            Alert.ERROR("Cannot write configuration file",e);
        }
    }
    
    public String toString () {  
        // for list of vectors used in song list

        return ARTIST+" - "+TITLE;

    }
    
    private Document parseSong() throws IOException {
        final int START = 0;
        final int BASE = 1;
        final int SHIFT = 2;
        final int EXTRA = 3;
        final int STOP = 4;
        final int NOT_CHORD = -1;
        final int CHORD = -2;

        LinkedChordList ChordList;
        this.ChordList = new LinkedChordList();
        
        MyStyleContext Styles = new MyStyleContext(); // Creating Stylesheet for document        
        doc = new DefaultStyledDocument(Styles); // Creating document with Stylesheet


        BufferedReader input = new BufferedReader(new FileReader(Config.getValue("library")+"/"+FILENAME));
        String line;
        StringTokenizer st;
        int linetype, lineN;
        lineN = 0;

        while ((line = input.readLine())!= null) {

            lineN++;
            st = new StringTokenizer(line);
            linetype = 0;
            String chLine = new String();
            while (st.hasMoreTokens()) {
                String tmp=st.nextToken();
                //String t = tmp.toUpperCase();
                String t = tmp;

                int state=START;
                Chord temp = new Chord();
                int p=0;
                while (p<t.length() && state != NOT_CHORD) {
                    char token=t.charAt(p);
                    switch(token) {
                        case 'A':
                        case 'B':
                        case 'C':
                        case 'D':
                        case 'E':
                        case 'F':
                        case 'G': if (state==START) {
                                    state = BASE;
                                    temp.base = t.substring(0,1);
                                    p++;
                                  }				 		
                                  else state=NOT_CHORD;
                                  break;
                        case '#':
                        case 'b': if (state==BASE) {
                                    state = SHIFT;
                                    temp.base = temp.base+token;
                                    p++;
                                  }
                                  else state=NOT_CHORD;
                                  break;
                        case 'm':
                        case 'M':
                        case '6':
                        case '7': if (state==BASE || state==SHIFT) {
                                        if (p+4<=t.length() && t.substring(p,p+4).compareTo("Maj7") == 0) {
                                            //System.out.println(t.substring(p,p+4));
                                            temp.extra = t.substring(p,p+4);
                                            p=p+4;
                                            state = EXTRA;
                                        }
                                        else if (p+3<=t.length() && t.substring(p,p+3).compareTo("Maj") == 0) {
                                            temp.extra = t.substring(p,p+3);
                                            p=p+3;
                                            state = EXTRA;
                                        }
                                        else if (p+2<=t.length() && 
                                                        (t.substring(p,p+2).compareTo("Mm") == 0 ||
                                                         t.substring(p,p+2).compareTo("m7") == 0)) {
                                            temp.extra = t.substring(p,p+2);
                                            p=p+2;
                                            state = EXTRA;
                                        }
                                        else if (p+1<=t.length() && (
                                                         t.substring(p,p+1).compareTo("m") == 0 ||
                                                         t.substring(p,p+1).compareTo("6") == 0 ||
                                                         t.substring(p,p+1).compareTo("7") == 0)) {
                                            temp.extra = t.substring(p,p+1);
                                            p=p+1;
                                            state = EXTRA;
                                        }
                                        else state = NOT_CHORD;
                                      }	
                                      else state = NOT_CHORD;
                                      break;
                            case 'a':
                            case 's': if ((state==BASE || state==SHIFT || state==EXTRA) &&
                                                        p+4<=t.length() && 
                                                        ((t.substring(p,p+4).compareTo("add5") == 0 ||
                                                            t.substring(p,p+4).compareTo("sus4") == 0))) {
                                            temp.extra = temp.extra + t.substring(p,p+4);
                                            p=p+4;
                                            state = STOP;
                                      }
                                      else state = NOT_CHORD;
                                      break;
                            default: state = NOT_CHORD;			
                    }
                } // chord catch
             
                if (state != NOT_CHORD && state != START) 
                 // IF chordTest passed AND line had been read
                {
                    linetype = CHORD;
                    line=line.replaceFirst(temp.toString(), temp.transpose(TRANSPOSE).toString());
                    
                    this.ChordList.add(temp); // adding chord to linked list
                }
                else 
                    linetype = NOT_CHORD;

            }
            
           // printing LINE to document:
            if (linetype == CHORD) 
               printDoc(doc, line, Styles.getStyle("CHORD"));
            else if (!line.startsWith("["))
               printDoc(doc, line, Styles.getStyle("TEXT"));
            
        }

        input.close();
  
        // SONG PARSED.
        this.ChordList.print();
        
        return doc;
    }

    private void printDoc(Document doc, String s, Style style) {
        try {
            doc.insertString(doc.getLength(), s+"\n", style);
        } catch (Exception e) { /* need some error msg here */ }
    }
    
    
}
